@@ -1,21 +1,13 @@
-from .base import (
-    ClientBaseWithDataset,
-    ClientBaseWithDatastack,
-    ClientBase,
-    BaseEncoder,
-    _api_versions,
-    _api_endpoints,
-    handle_response,
-)
-from .auth import AuthClient
-from .endpoints import annotation_common, annotation_api_versions
-from .infoservice import InfoServiceClientV2
-
 import json
+from typing import Iterable, List, Mapping
+
 import numpy as np
-from datetime import date, datetime
 import pandas as pd
-from typing import Iterable, Mapping
+
+from .auth import AuthClient
+from .base import BaseEncoder, ClientBase, _api_endpoints, handle_response
+from .endpoints import annotation_api_versions, annotation_common
+from .tools import stage
 
 SERVER_KEY = "ae_server_address"
 
@@ -30,6 +22,7 @@ def AnnotationClient(
     max_retries=None,
     pool_maxsize=None,
     pool_block=None,
+    over_client=None,
 ):
     """Factory for returning AnnotationClient
 
@@ -86,6 +79,7 @@ def AnnotationClient(
             max_retries=max_retries,
             pool_maxsize=pool_maxsize,
             pool_block=pool_block,
+            over_client=over_client,
         )
     else:
         return AnnoClient(
@@ -99,6 +93,7 @@ def AnnotationClient(
             max_retries=max_retries,
             pool_maxsize=pool_maxsize,
             pool_block=pool_block,
+            over_client=over_client,
         )
 
 
@@ -115,6 +110,8 @@ class AnnotationClientV2(ClientBase):
         max_retries=None,
         pool_maxsize=None,
         pool_block=None,
+        over_client=None,
+        schema_client=None,
     ):
         super(AnnotationClientV2, self).__init__(
             server_address,
@@ -126,15 +123,20 @@ class AnnotationClientV2(ClientBase):
             max_retries=max_retries,
             pool_maxsize=pool_maxsize,
             pool_block=pool_block,
+            over_client=over_client,
         )
 
         self._aligned_volume_name = aligned_volume_name
+        if schema_client is None:
+            self._schema_client = self.fc.schema
+        else:
+            self._schema_client = schema_client
 
     @property
     def aligned_volume_name(self):
         return self._aligned_volume_name
 
-    def get_tables(self, aligned_volume_name=None):
+    def get_tables(self, aligned_volume_name: str = None):
         """Gets a list of table names for a aligned_volume_name
 
         Parameters
@@ -154,11 +156,10 @@ class AnnotationClientV2(ClientBase):
         endpoint_mapping = self.default_url_mapping
         endpoint_mapping["aligned_volume_name"] = aligned_volume_name
         url = self._endpoints["tables"].format_map(endpoint_mapping)
-        print(url)
         response = self.session.get(url)
         return handle_response(response)
 
-    def get_annotation_count(self, table_name: str, aligned_volume_name=None):
+    def get_annotation_count(self, table_name: str, aligned_volume_name: str = None):
         """Get number of annotations in a table
 
         Parameters
@@ -186,7 +187,7 @@ class AnnotationClientV2(ClientBase):
         response = self.session.get(url)
         return handle_response(response)
 
-    def get_table_metadata(self, table_name: str, aligned_volume_name=None):
+    def get_table_metadata(self, table_name: str, aligned_volume_name: str = None):
         """Get metadata about a table
 
         Parameters
@@ -210,7 +211,6 @@ class AnnotationClientV2(ClientBase):
         endpoint_mapping["table_name"] = table_name
 
         url = self._endpoints["table_info"].format_map(endpoint_mapping)
-
         response = self.session.get(url)
         metadata_d = handle_response(response)
         vx = metadata_d.pop("voxel_resolution_x")
@@ -219,9 +219,8 @@ class AnnotationClientV2(ClientBase):
         metadata_d["voxel_resolution"] = [vx, vy, vz]
         return metadata_d
 
-    def delete_table(self, table_name: str, aligned_volume_name=None):
-        """Marks a table for deletion
-        requires super admin priviledges
+    def delete_table(self, table_name: str, aligned_volume_name: str = None):
+        """Marks a table for deletion requires super admin privileges
 
         Parameters
         ----------
@@ -250,14 +249,15 @@ class AnnotationClientV2(ClientBase):
 
     def create_table(
         self,
-        table_name,
-        schema_name,
-        description,
-        voxel_resolution,
-        reference_table=None,
-        flat_segmentation_source=None,
-        user_id=None,
-        aligned_volume_name=None,
+        table_name: str,
+        schema_name: str,
+        description: str,
+        voxel_resolution: List[float],
+        reference_table: str = None,
+        track_target_id_updates: bool = None,
+        flat_segmentation_source: str = None,
+        user_id: int = None,
+        aligned_volume_name: str = None,
     ):
         """Creates a new data table based on an existing schema
 
@@ -282,7 +282,11 @@ class AnnotationClientV2(ClientBase):
         reference_table: str or None
             If the schema you are using is a reference schema
             Meaning it is an annotation of another annotation.
-            Then you need to specify what table those annotations are in.
+            Then you need to specify what the target table
+            those annotations are in.
+        track_target_id_updates: bool or None
+            Indicates whether to automatically update reference table's foreign key
+            if target annotation table row is updated.
         flat_segmentation_source: str or None
             the source to a flat segmentation that corresponds to this table
             i.e. precomputed:\\gs:\\mybucket\this_tables_annotation
@@ -298,6 +302,12 @@ class AnnotationClientV2(ClientBase):
         json
             Response JSON
 
+        Examples
+        --------
+        Basic annotation table:
+        description = "Some description about the table"
+        voxel_res = [4,4,40]
+        client.create_table("some_synapse_table", "synapse", description, voxel_res)
         """
         if aligned_volume_name is None:
             aligned_volume_name = self.aligned_volume_name
@@ -315,7 +325,10 @@ class AnnotationClientV2(ClientBase):
         if user_id is not None:
             metadata["user_id"] = user_id
         if reference_table is not None:
-            metadata["reference_table"] = reference_table
+            metadata["table_metadata"] = {
+                "reference_table": reference_table,
+                "track_target_id_updates": track_target_id_updates,
+            }
         if flat_segmentation_source is not None:
             metadata["flat_segmentation_source"] = flat_segmentation_source
 
@@ -328,7 +341,12 @@ class AnnotationClientV2(ClientBase):
         response = self.session.post(url, json=data)
         return handle_response(response, as_json=False)
 
-    def get_annotation(self, table_name, annotation_ids, aligned_volume_name=None):
+    def get_annotation(
+        self,
+        table_name: str,
+        annotation_ids: (int or Iterable),
+        aligned_volume_name: str = None,
+    ):
         """Retrieve an annotation or annotations by id(s) and table name.
 
         Parameters
@@ -361,8 +379,13 @@ class AnnotationClientV2(ClientBase):
         response = self.session.get(url, params=params)
         return handle_response(response)
 
-    def post_annotation(self, table_name, data, aligned_volume_name=None):
-        """Post one or more new annotations to a table in the AnnotationEngine
+    def post_annotation(
+        self, table_name: str, data: (dict or List), aligned_volume_name: str = None
+    ):
+        """Post one or more new annotations to a table in the AnnotationEngine.
+        All inserted annotations will be marked as 'valid'. To invalidate
+        annotations refer to 'update_annotation', 'update_annotation_df'
+        and 'delete_annotation' methods.
 
         Parameters
         ----------
@@ -407,7 +430,7 @@ class AnnotationClientV2(ClientBase):
     def process_position_columns(
         df: pd.DataFrame, position_columns: (Iterable[str] or Mapping[str, str] or None)
     ):
-        """process a dataframe into a list of dictionaries, nesting thing
+        """Process a dataframe into a list of dictionaries, nesting thing
 
         Args:
             df (pd.DataFrame): dataframe to process
@@ -437,7 +460,10 @@ class AnnotationClientV2(ClientBase):
         position_columns: (Iterable[str] or Mapping[str, str] or None),
         aligned_volume_name=None,
     ):
-        """Post one or more new annotations to a table in the AnnotationEngine
+        """Post one or more new annotations to a table in the AnnotationEngine.
+        All inserted annotations will be marked as 'valid'. To invalidate
+        annotations see 'update_annotation', 'update_annotation_df'
+        and 'delete_annotation' methods.
 
         Parameters
         ----------
@@ -473,10 +499,17 @@ class AnnotationClientV2(ClientBase):
             table_name, data, aligned_volume_name=aligned_volume_name
         )
 
-    def update_annotation(self, table_name, data, aligned_volume_name=None):
-        """Update one or more new annotations to a table in the AnnotationEngine
-        Note update is implemented by deleting the old annotation
-        and inserting a new annotation, which will receive a new ID.
+    def update_annotation(
+        self, table_name: str, data: (dict or List), aligned_volume_name: str = None
+    ):
+        """Update one or more new annotations to a table in the AnnotationEngine.
+        Updating is implemented by invalidating the old annotation
+        and inserting a new annotation row, which will receive a new primary key ID.
+
+        Notes
+        -----
+        If annotations ids were user provided upon insertion the database will
+        autoincrement from the current max id in the table.
 
         Parameters
         ----------
@@ -507,11 +540,15 @@ class AnnotationClientV2(ClientBase):
         try:
             iter(data)
         except TypeError:
-            annotation_ids = [data]
+            data = [data]
 
         data = {"annotations": data}
 
-        response = self.session.put(url, json=data)
+        response = self.session.put(
+            url,
+            data=json.dumps(data, cls=BaseEncoder),
+            headers={"Content-Type": "application/json"},
+        )
         return handle_response(response)
 
     def update_annotation_df(
@@ -521,7 +558,14 @@ class AnnotationClientV2(ClientBase):
         position_columns: (Iterable[str] or Mapping[str, str] or None),
         aligned_volume_name=None,
     ):
-        """Update one or more  annotations to a table in the AnnotationEngine using a dataframe as format
+        """Update one or more annotations to a table in the AnnotationEngine using a
+        dataframe as format. Updating is implemented by invalidating the old annotation
+        and inserting a new annotation row, which will receive a new primary key ID.
+
+        Notes
+        -----
+        If annotations ids were user provided upon insertion the database will
+        autoincrement from the current max id in the table.
 
         Parameters
         ----------
@@ -557,10 +601,14 @@ class AnnotationClientV2(ClientBase):
             table_name, data, aligned_volume_name=aligned_volume_name
         )
 
-    def delete_annotation(self, table_name, annotation_ids, aligned_volume_name=None):
-        """Update one or more new annotations to a table in the AnnotationEngine
-        Note update is implemented by deleting the old annotation
-        and inserting a new annotation, which will receive a new ID.
+    def delete_annotation(
+        self,
+        table_name: str,
+        annotation_ids: (dict or List),
+        aligned_volume_name: str = None,
+    ):
+        """Delete one or more annotations in a table. Annotations that are
+        deleted are recorded as 'non-valid' but are not physically removed from the table.
 
         Parameters
         ----------
@@ -600,6 +648,97 @@ class AnnotationClientV2(ClientBase):
         )
         return handle_response(response)
 
+    def stage_annotations(
+        self,
+        table_name=None,
+        schema_name=None,
+        update=False,
+        id_field=False,
+        table_resolution=None,
+        annotation_resolution=None,
+    ):
+        """
+        Get a StagedAnnotations object to help produce correctly formatted annotations for a given table or schema.
+        StagedAnnotation objects can be uploaded directly with `upload_staged_annotations`.
+
+        Parameters
+        ----------
+        table_name : str, optional
+            Table name to stage annotations for, by default None.
+        schema_name : str, optional
+            Schema name to use to make annotations. Only needed if the table_name is not set, by default None
+        update : bool, optional
+            Set to True if individual annotations are going to be updated, by default False.
+        id_field : bool, optional
+            Set to True if id fields are to be specified. Not needed if update is True, which always needs id fields. Optional, by default False
+        table_resolution : list-like or None, optional
+            Voxel resolution of spatial points in the table in nanometers. This is found automatically from the info service if a table name is provided, by default None.
+            If annotation_resolution is also set, this allows points to be scaled correctly for the table.
+        annotation_resolution : list-like, optional
+            Voxel resolution of spatial points provided by the user when creating annotations. If the table resolution is also available (manually or from the info service),
+            annotations are correctly rescaled for the volume. By default, None.
+        """
+
+        if table_name is not None:
+            obj_name = table_name
+            table_meta = self.get_table_metadata(table_name)
+            schema_name = table_meta["schema_type"]
+            table_resolution = table_meta["voxel_resolution"]
+        else:
+            if schema_name is None:
+                raise ValueError("Must specify either table name or schema name")
+            obj_name = schema_name
+
+        schema = self._schema_client.schema_definition(schema_name)
+        return stage.StagedAnnotations(
+            schema,
+            name=obj_name,
+            update=update,
+            id_field=id_field,
+            annotation_resolution=annotation_resolution,
+            table_resolution=table_resolution,
+            table_name=table_name,
+        )
+
+    def upload_staged_annotations(
+        self,
+        staged_annos: stage.StagedAnnotations,
+        aligned_volume_name: str = None,
+    ):
+        """
+        Upload annotations directly from an Annotation Guide object.
+        This method uses the options specified in the object, including table name and if the annotation is an update or not.
+
+        Parameters
+        ----------
+        staged_annos : guide.AnnotationGuide
+            AnnotationGuide object with a specified table name and a collection of annotations already filled in.
+        aligned_volume_name : str or None, optional
+            Name of the aligned_volume. If None, uses the one specified in the client.
+
+        Returns
+        -------
+        List or dict
+            If new annotations are posted, a list of ids.
+            If annotations are being updated, a dictionary with the mapping from old ids to new ids.
+        """
+        if staged_annos.table_name is None:
+            raise ValueError(
+                "Only annotation guide objects with a specified table name can be used here"
+            )
+        if staged_annos.is_update:
+            return self.update_annotation(
+                staged_annos.table_name,
+                staged_annos.annotation_list,
+                aligned_volume_name=aligned_volume_name,
+            )
+        else:
+            return self.post_annotation(
+                staged_annos.table_name,
+                staged_annos.annotation_list,
+                aligned_volume_name=aligned_volume_name,
+            )
+
 
 client_mapping = {
     2: AnnotationClientV2,
