@@ -2,15 +2,17 @@ import urllib
 import requests
 import json
 import logging
+logger = logging.getLogger(__name__)
 import webbrowser
+
 from .session_config import patch_session
 import numpy as np
 import datetime
-
+import pandas as pd
 
 class BaseEncoder(json.JSONEncoder):
     def default(self, obj):
-        if isinstance(obj, np.ndarray):
+        if isinstance(obj, np.ndarray) or isinstance(obj, pd.Series):
             return obj.tolist()
         if isinstance(obj, np.uint64):
             return int(obj)
@@ -68,7 +70,7 @@ def _raise_for_status(r):
         raise requests.HTTPError(http_error_msg, response=r)
     warning = r.headers.get("Warning")
     if warning:
-        logging.warning(warning)
+        logger.warning(warning)
 
 
 def handle_response(response, as_json=True):
@@ -159,6 +161,7 @@ class ClientBase(object):
         max_retries=None,
         pool_maxsize=None,
         pool_block=None,
+        over_client=None,
     ):
         self._server_address = server_address
         self._default_url_mapping = {server_name: self._server_address}
@@ -183,6 +186,11 @@ class ClientBase(object):
 
         self._api_version = api_version
         self._endpoints = endpoints
+        self._fc = over_client
+
+    @property
+    def fc(self):
+        return self._fc
 
     @property
     def default_url_mapping(self):
@@ -216,6 +224,7 @@ class ClientBaseWithDataset(ClientBase):
         max_retries=None,
         pool_maxsize=None,
         pool_block=None,
+        over_client=None,
     ):
 
         super(ClientBaseWithDataset, self).__init__(
@@ -228,6 +237,7 @@ class ClientBaseWithDataset(ClientBase):
             max_retries=max_retries,
             pool_maxsize=pool_maxsize,
             pool_block=pool_block,
+            over_client=over_client,
         )
         self._dataset_name = dataset_name
 
@@ -249,6 +259,7 @@ class ClientBaseWithDatastack(ClientBase):
         max_retries=None,
         pool_maxsize=None,
         pool_block=None,
+        over_client=None,
     ):
 
         super(ClientBaseWithDatastack, self).__init__(
@@ -261,6 +272,7 @@ class ClientBaseWithDatastack(ClientBase):
             max_retries=max_retries,
             pool_maxsize=pool_maxsize,
             pool_block=pool_block,
+            over_client=over_client,
         )
         self._datastack_name = datastack_name
 
