@@ -56,6 +56,7 @@ def package_timestamp(timestamp, name="timestamp"):
 
 def root_id_int_list_check(
     root_id,
+    make_unique=False,
 ):
     if isinstance(root_id, int) or isinstance(root_id, np.uint64):
         root_id = [root_id]
@@ -67,7 +68,10 @@ def root_id_int_list_check(
                 "When passing a string for 'root_id' make sure the string can be converted to a uint64"
             )
     elif isinstance(root_id, np.ndarray) or isinstance(root_id, list):
-        root_id = np.unique(root_id).astype(np.uint64)
+        if make_unique:
+            root_id = np.unique(root_id).astype(np.uint64)
+        else:
+            root_id = np.array(root_id, dtype=np.uint64)
     else:
         raise ValueError("root_id has to be list or uint64")
 
@@ -84,6 +88,7 @@ def ChunkedGraphClient(
     max_retries=None,
     pool_maxsize=None,
     pool_block=None,
+    over_client=None,
 ):
     if server_address is None:
         server_address = default_global_server_address
@@ -115,6 +120,7 @@ def ChunkedGraphClient(
         max_retries=max_retries,
         pool_maxsize=pool_maxsize,
         pool_block=pool_block,
+        over_client=over_client,
     )
 
 
@@ -134,6 +140,7 @@ class ChunkedGraphClientV1(ClientBase):
         max_retries=None,
         pool_maxsize=None,
         pool_block=None,
+        over_client=None,
     ):
         super(ChunkedGraphClientV1, self).__init__(
             server_address,
@@ -145,6 +152,7 @@ class ChunkedGraphClientV1(ClientBase):
             max_retries=max_retries,
             pool_maxsize=pool_maxsize,
             pool_block=pool_block,
+            over_client=over_client,
         )
         self._default_url_mapping["table_id"] = table_name
         self._default_timestamp = timestamp
@@ -264,6 +272,47 @@ class ChunkedGraphClientV1(ClientBase):
 
         return handle_response(response)
 
+    def get_user_operations(
+        self,
+        user_id: int,
+        timestamp_start: datetime.datetime,
+        include_undo: bool = True,
+        timestamp_end: datetime.datetime = None,
+    ):
+        """get operation details for a user_id
+
+        Args:
+            user_id (int): userID to query (use 0 for all users [admin only])
+            timestamp_start (datetime.datetime, optional): timestamp to start filter (UTC).
+            include_undo (bool, optional): whether to include undos. Defaults to True.
+            timestamp_end (datetime.datetime, optional): timestamp to end filter (UTC). Defaults to now.
+        """
+        endpoint_mapping = self.default_url_mapping
+
+        url = self._endpoints["user_operations"].format_map(endpoint_mapping)
+        params = {"include_undo": include_undo}
+
+        if user_id > 0:
+            params = {"user_id": user_id}
+        if timestamp_start is not None:
+            params.update(
+                package_timestamp(
+                    self._process_timestamp(timestamp_start), "start_time"
+                )
+            )
+        if timestamp_end is not None:
+            params.update(
+                package_timestamp(self._process_timestamp(timestamp_end), "end_time")
+            )
+        response = self.session.get(url, params=params)
+
+        d = handle_response(response)
+        df = pd.DataFrame(d)
+        df["timestamp"] = df["timestamp"].map(
+            lambda x: datetime.datetime.fromtimestamp(x / 1000, pytz.UTC)
+        )
+        return df
+
     def get_tabular_change_log(self, root_ids, filtered=True):
         """Get a detailed changelog for neurons
 
@@ -348,6 +397,88 @@ class ChunkedGraphClientV1(ClientBase):
         )
         handle_response(response)
 
+    def preview_split(
+        self,
+        source_points,
+        sink_points,
+        root_id,
+        source_supervoxels=None,
+        sink_supervoxels=None,
+        return_additional_ccs=False,
+    ):
+        """Get supervoxel connected components from a preview multicut split.
+
+        Parameters
+        ----------
+        source_points : array or list
+            Nx3 list or array of 3d points in nm coordinates for source points (red).
+        sink_points : array or list
+            Mx3 list or array of 3d points in nm coordinates for sink points (blue).
+        root_id : int
+            root id of object to do split preview.
+        source_supervoxels : array, list or None, optional
+            If providing source supervoxels, an N-length array of supervoxel ids or Nones matched to source points. If None, treats as a full array of Nones. By default None
+        sink_supervoxels : array, list or None, optional
+            If providing sink supervoxels, an M-length array of supervoxel ids or Nones matched to source points. If None, treats as a full array of Nones. By default None
+        return_additional_ccs : bool, optional
+            If True, returns any additional connected components beyond the ones with source and sink points. In most situations, this can be ignored. By default, False.
+
+        Returns
+        -------
+        source_connected_component
+            List of supervoxel ids in the component with the most source points.
+        sink_connected_component
+            List of supervoxel ids in the component with the most sink points.
+        successful_split
+            Boolean value that is True if the split worked.
+        other_connected_components (optional)
+            List of lists of supervoxel ids for any other resulting connected components. Only returned if `return_additional_ccs` is True.
+        """
+        endpoint_mapping = self.default_url_mapping
+        url = self._endpoints["preview_split"].format_map(endpoint_mapping)
+
+        categories = ["sources", "sinks"]
+        pts = [source_points, sink_points]
+        svs = [source_supervoxels, sink_supervoxels]
+        for pt_list, sv_list in zip(pts, svs):
+            if sv_list is not None:
+                if len(pt_list) != len(sv_list):
+                    raise ValueError(
+                        "If supervoxels are provided, they must have the same length as points"
+                    )
+
+        data = {}
+        for cat, pt_list, sv_list in zip(categories, pts, svs):
+            if sv_list is None:
+                sv_list = [None] * len(pt_list)
+            sv_list = [x if x is not None else root_id for x in sv_list]
+
+            out = []
+            for svid, pt in zip(sv_list, pt_list):
+                out.append([svid, pt[0], pt[1], pt[2]])
+
+            data[cat] = out
+
+        response = self.session.post(
+            url,
+            data=json.dumps(data, cls=BaseEncoder),
+            headers={"Content-Type": "application/json"},
+        )
+        r = handle_response(response)
+        source_cc = r["supervoxel_connected_components"][0]
+        sink_cc = r["supervoxel_connected_components"][1]
+        if len(r["supervoxel_connected_components"]) == 2:
+            other_ccs = []
+        else:
+            other_ccs = r["supervoxel_connected_components"][2:]
+
+        success = not r["illegal_split"]
+
+        if return_additional_ccs:
+            return source_cc, sink_cc, success, other_ccs
+        else:
+            return source_cc, sink_cc, success
+
     def get_children(self, node_id):
         """Get the children of a node in the hierarchy
 
@@ -525,7 +656,7 @@ class ChunkedGraphClientV1(ClientBase):
         dict
             Dictionary describing the lineage graph and operations for the root id.
         """
-        root_id = root_id_int_list_check(root_id)
+        root_id = root_id_int_list_check(root_id, make_unique=True)
 
         endpoint_mapping = self.default_url_mapping
 
@@ -559,7 +690,7 @@ class ChunkedGraphClientV1(ClientBase):
         np.ndarray
             1d array with all latest successors
         """
-        root_id = root_id_int_list_check(root_id)
+        root_id = root_id_int_list_check(root_id, make_unique=True)
 
         timestamp_past = self.get_root_timestamps(root_id).min()
 
@@ -590,7 +721,7 @@ class ChunkedGraphClientV1(ClientBase):
         np.ndarray
             1d array with all latest successors
         """
-        root_id = root_id_int_list_check(root_id)
+        root_id = root_id_int_list_check(root_id, make_unique=True)
 
         timestamp_future = self.get_root_timestamps(root_id).max()
 
@@ -615,9 +746,9 @@ class ChunkedGraphClientV1(ClientBase):
             timestamp (datetime.dateime, optional): timestamp to check whether these IDs are valid root_ids. Defaults to None (assumes now).
 
         Returns:
-            np.array[np.Boolean]: boolean array of whether these are valid root_ids
+            np.array[bool]: boolean array of whether these are valid root_ids
         """
-        root_ids = root_id_int_list_check(root_ids)
+        root_ids = root_id_int_list_check(root_ids, make_unique=False)
 
         endpoint_mapping = self.default_url_mapping
         url = self._endpoints["is_latest_roots"].format_map(endpoint_mapping)
@@ -634,7 +765,58 @@ class ChunkedGraphClientV1(ClientBase):
                 url, data=json.dumps(data, cls=BaseEncoder), params=query_d
             )
         )
-        return np.array(r["is_latest"], np.bool)
+        return np.array(r["is_latest"], bool)
+
+    def is_valid_nodes(self, node_ids, start_timestamp=None, end_timestamp=None):
+        """Check whether nodes are valid for given timestamp range
+
+        Valid is defined as existing in the chunkedgraph. This makes no statement
+        about these IDs being roots, supervoxel or anything in-between. It also
+        does not take into account whether a root id has since been edited.
+
+        Parameters
+        ----------
+            node ids ([type]): node ids to check
+            start_timestamp (datetime.dateime, optional): timestamp to check whether these IDs were valid after this timestamp. Defaults to None (assumes now).
+            end_timestamp (datetime.dateime, optional): timestamp to check whether these IDs were valid before this timestamp. Defaults to None (assumes now).
+
+        Returns:
+            np.array[np.Boolean]: boolean array of whether these are valid IDs
+        """
+        node_ids = root_id_int_list_check(node_ids, make_unique=False)
+
+        endpoint_mapping = self.default_url_mapping
+        url = self._endpoints["valid_nodes"].format_map(endpoint_mapping)
+
+        if end_timestamp is None:
+            end_timestamp = self._default_timestamp
+
+        if start_timestamp is None:
+            start_timestamp = datetime.datetime(2000, 1, 1)
+
+        if start_timestamp is not None:
+            query_d = package_timestamp(
+                self._process_timestamp(start_timestamp), name="start_timestamp"
+            )
+        else:
+            query_d = {}
+
+        if end_timestamp is not None:
+            query_d.update(
+                package_timestamp(
+                    self._process_timestamp(end_timestamp), name="end_timestamp"
+                )
+            )
+
+        data = {"node_ids": node_ids}
+        r = handle_response(
+            self.session.get(
+                url, data=json.dumps(data, cls=BaseEncoder), params=query_d
+            )
+        )
+        valid_ids = np.array(r["valid_roots"], np.uint64)
+
+        return np.isin(node_ids, valid_ids)
 
     def get_root_timestamps(self, root_ids):
         """Retrieves timestamps when roots where created.
@@ -648,7 +830,7 @@ class ChunkedGraphClientV1(ClientBase):
         -------
 
         """
-        root_ids = root_id_int_list_check(root_ids)
+        root_ids = root_id_int_list_check(root_ids, make_unique=False)
 
         endpoint_mapping = self.default_url_mapping
         url = self._endpoints["root_timestamps"].format_map(endpoint_mapping)
@@ -680,7 +862,7 @@ class ChunkedGraphClientV1(ClientBase):
             Dict with keys `future_id_map` and `past_id_map`. Each is a dict whose keys are the supplied root ids and whose values
             are the list of related root ids at the past/future time stamp.
         """
-        root_ids = root_id_int_list_check(root_ids)
+        root_ids = root_id_int_list_check(root_ids, make_unique=True)
 
         endpoint_mapping = self.default_url_mapping
 
