@@ -1,42 +1,63 @@
-import logging
-from cachetools import cached, TTLCache
-from typing import ValuesView
-
-from numpy.lib.function_base import iterable
-from numpy.lib.twodim_base import _trilu_indices_form_dispatcher
-import caveclient
+import re
+import warnings
+import pytz
+import pandas as pd
+from IPython.display import HTML
+from .timeit import TimeIt
+from typing import Union, Iterable
+import itertools
+import pyarrow as pa
+from datetime import datetime, timezone
+import numpy as np
+import json
+from .endpoints import materialization_api_versions, materialization_common
+from .auth import AuthClient
 from .base import (
-    ClientBaseWithDataset,
-    ClientBaseWithDatastack,
     ClientBase,
     BaseEncoder,
-    _api_versions,
     _api_endpoints,
     handle_response,
 )
-from .auth import AuthClient
-from .endpoints import materialization_api_versions, materialization_common
-from .infoservice import InfoServiceClientV2
-import requests
-import time
-import json
-import numpy as np
-from datetime import date, datetime, timezone, tzinfo
-import pyarrow as pa
-import itertools
-from collections.abc import Iterable
-from typing import Union
-from .timeit import TimeIt
-from IPython.display import HTML
-import pandas as pd
-import pytz
-import warnings
+from cachetools import cached, TTLCache
+import logging
+
+logger = logging.getLogger(__name__)
 
 SERVER_KEY = "me_server_address"
 
 
+def convert_position_columns(df, given_resolution, desired_resolution):
+    """function to take a dataframe with x,y,z position columns and convert
+    them to the desired resolution from the given resolution
+
+    Args:
+        df (pd.DataFrame): dataframe to alter
+        given_resolution (Iterable[float]): what the given resolution is
+        desired_resoultion (Iterable[float]): what the desired resolution is
+
+    Returns:
+        pd.DataFrame: [description]
+    """
+    gr = np.array(given_resolution)
+    dr = np.array(desired_resolution)
+    sf = gr / dr
+    posmap = {"x": 0, "y": 1, "z": 2}
+    if np.all(sf == 1):
+        return df
+    else:
+        grps = itertools.groupby(df.columns, key=lambda x: x[:-2])
+        for _, g in grps:
+            gl = list(g)
+            t = "".join([k[-1:] for k in gl])
+            if t == "xyz":
+                for col in gl:
+                    df[col] = df[col] * sf[posmap[col[-1]]]
+
+    return df
+
+
 def concatenate_position_columns(df, inplace=False):
-    """function to take a dataframe with xyz position columns and replace them
+    """function to take a dataframe with x,y,z position columns and replace them
     with one column per position with an xyz numpy array.  Edits occur
 
     Args:
@@ -64,15 +85,29 @@ def concatenate_position_columns(df, inplace=False):
 
 
 def convert_timestamp(ts: datetime):
+    if ts == "now":
+        ts = datetime.utcnow()
+
     if isinstance(ts, datetime):
         if ts.tzinfo is None:
-            return pytz.UTC.localize(ts)
+            return pytz.UTC.localize(dt=ts)
         else:
             return ts.astimezone(timezone.utc)
+    elif isinstance(ts, float):
+        return datetime.fromtimestamp(ts)
+    elif ts is None:
+        return pd.Timestamp.max.to_pydatetime()
     dt = datetime.strptime(ts, "%Y-%m-%dT%H:%M:%S.%f")
     return dt.replace(tzinfo=timezone.utc)
 
 
+def string_format_timestamp(ts):
+    if isinstance(ts, datetime):
+        return datetime.strftime(ts, "%Y-%m-%dT%H:%M:%S.%f")
+    else:
+        return ts
+
+
 def MaterializationClient(
     server_address,
     datastack_name=None,
@@ -85,6 +120,8 @@ def MaterializationClient(
     max_retries=None,
     pool_maxsize=None,
     pool_block=None,
+    desired_resolution=None,
+    over_client=None,
 ):
     """Factory for returning AnnotationClient
 
@@ -106,6 +143,9 @@ def MaterializationClient(
         default synapse table for queries
     version : default version to query
         if None will default to latest version
+    desired_resolution : Iterable[float] or None, optional
+        If given, should be a list or array of the desired resolution you want queries returned in
+        useful for materialization queries.
 
     Returns
     -------
@@ -141,6 +181,8 @@ def MaterializationClient(
         max_retries=max_retries,
         pool_maxsize=pool_maxsize,
         pool_block=pool_block,
+        over_client=over_client,
+        desired_resolution=desired_resolution,
     )
 
 
@@ -160,6 +202,8 @@ class MaterializatonClientV2(ClientBase):
         max_retries=None,
         pool_maxsize=None,
         pool_block=None,
+        over_client=None,
+        desired_resolution=None,
     ):
         super(MaterializatonClientV2, self).__init__(
             server_address,
@@ -171,13 +215,18 @@ class MaterializatonClientV2(ClientBase):
             max_retries=max_retries,
             pool_maxsize=pool_maxsize,
             pool_block=pool_block,
+            over_client=over_client,
         )
         self._datastack_name = datastack_name
         if version is None:
             version = self.most_recent_version()
         self._version = version
-        self.cg_client = cg_client
+        if cg_client is None:
+            self.cg_client = self.fc.chunkedgraph
+        else:
+            self.cg_client = cg_client
         self.synapse_table = synapse_table
+        self.desired_resolution = desired_resolution
 
     @property
     def datastack_name(self):
@@ -350,6 +399,7 @@ class MaterializatonClientV2(ClientBase):
             md["expires_on"] = convert_timestamp(md["expires_on"])
         return d
 
+    @cached(cache=TTLCache(maxsize=100, ttl=60 * 60 * 12))
     def get_table_metadata(
         self, table_name: str, datastack_name=None, version: int = None
     ):
@@ -388,49 +438,6 @@ class MaterializatonClientV2(ClientBase):
         metadata_d["voxel_resolution"] = [vx, vy, vz]
         return metadata_d
 
-    # def get_annotation(self, table_name, annotation_ids,
-    #                    materialization_version=None,
-    #                    datastack_name=None):
-    #     """ Retrieve an annotation or annotations by id(s) and table name.
-
-    #     Parameters
-    #     ----------
-    #     table_name : str
-    #         Name of the table
-    #     annotation_ids : int or iterable
-    #         ID or IDS of the annotation to retreive
-    #     materialization_version: int or None
-    #         materialization version to use
-    #         If None, uses the one specified in the client
-    #     datastack_name : str or None, optional
-    #         Name of the datastack_name.
-    #         If None, uses the one specified in the client.
-    #     Returns
-    #     -------
-    #     list
-    #         Annotation data
-    #     """
-    #     if materialization_version is None:
-    #         materialization_version = self.version
-    #     if datastack_name is None:
-    #         datastack_name = self.datastack_name
-
-    #     endpoint_mapping = self.default_url_mapping
-    #     endpoint_mapping["datastack_name"] = datastack_name
-    #     endpoint_mapping["table_name"] = table_name
-    #     endpoint_mapping["version"] = materialization_version
-    #     url = self._endpoints["annotations"].format_map(endpoint_mapping)
-    #     try:
-    #         iter(annotation_ids)
-    #     except TypeError:
-    #         annotation_ids = [annotation_ids]
-
-    #     params = {
-    #         'annotation_ids': ",".join([str(a) for a in annotation_ids])
-    #     }
-    #     response = self.session.get(url, params=params)
-    #     self.raise_for_status(response)
-    #     return response.json()
     def _format_query_components(
         self,
         datastack_name,
@@ -485,6 +492,32 @@ class MaterializatonClientV2(ClientBase):
 
         return url, data, query_args, encoding
 
+    def _resolve_merge_reference(
+        self, merge_reference, table, datastack_name, materialization_version
+    ):
+        if merge_reference:
+            md = self.get_table_metadata(
+                table_name=table,
+                datastack_name=datastack_name,
+                version=materialization_version,
+            )
+            if md["reference_table"] is None:
+                target_table = None
+            else:
+                if len(md["reference_table"]) == 0:
+                    target_table = None
+                else:
+                    target_table = md["reference_table"]
+        else:
+            target_table = None
+        if target_table is not None:
+            tables = [[table, "target_id"], [md["reference_table"], "id"]]
+            suffixes = ("", "ref")
+        else:
+            tables = [table]
+            suffixes = None
+        return tables, suffixes
+
     def query_table(
         self,
         table: str,
@@ -492,7 +525,6 @@ class MaterializatonClientV2(ClientBase):
         filter_out_dict=None,
         filter_equal_dict=None,
         filter_spatial_dict=None,
-        join_args=None,
         select_columns=None,
         offset: int = None,
         limit: int = None,
@@ -501,6 +533,9 @@ class MaterializatonClientV2(ClientBase):
         split_positions: bool = False,
         materialization_version: int = None,
         timestamp: datetime = None,
+        metadata: bool = True,
+        merge_reference: bool = True,
+        desired_resolution: Iterable = None,
     ):
         """generic query on materialization tables
 
@@ -536,6 +571,14 @@ class MaterializatonClientV2(ClientBase):
                 If None defaults to one specified in client.
             timestamp (datetime.datetime, optional): timestamp to query
                 If passsed will do a live query. Error if also passing a materialization version
+            metadata: (bool, optional) : toggle to return metadata (default True)
+                If True (and return_df is also True), return table and query metadata in the df.attr dictionary.
+            merge_reference: (bool, optional) : toggle to automatically join reference table
+                If True, metadata will be queries and if its a reference table it will perform a join
+                on the reference table to return the rows of that
+            desired_resolution: (Iterable[float], Optional) : desired resolution you want all spatial points returned in
+                If None, defaults to one specified in client, if that is None then points are returned
+                as stored in the table and should be in the resolution specified in the table metadata
         Returns:
         pd.DataFrame: a pandas dataframe of results of query
 
@@ -551,25 +594,31 @@ class MaterializatonClientV2(ClientBase):
                     filter_out_dict=filter_out_dict,
                     filter_equal_dict=filter_equal_dict,
                     filter_spatial_dict=filter_spatial_dict,
-                    join_args=join_args,
                     select_columns=select_columns,
                     offset=offset,
                     limit=limit,
                     datastack_name=datastack_name,
                     split_positions=split_positions,
                     post_filter=True,
+                    metadata=metadata,
+                    merge_reference=merge_reference,
+                    desired_resolution=desired_resolution,
                 )
         if materialization_version is None:
             materialization_version = self.version
         if datastack_name is None:
             datastack_name = self.datastack_name
 
+        tables, suffixes = self._resolve_merge_reference(
+            merge_reference, table, datastack_name, materialization_version
+        )
+
         url, data, query_args, encoding = self._format_query_components(
             datastack_name,
             materialization_version,
-            [table],
+            tables,
             select_columns,
-            None,
+            suffixes,
             {table: filter_in_dict} if filter_in_dict is not None else None,
             {table: filter_out_dict} if filter_out_dict is not None else None,
             {table: filter_equal_dict} if filter_equal_dict is not None else None,
@@ -593,6 +642,36 @@ class MaterializatonClientV2(ClientBase):
                 warnings.simplefilter(action="ignore", category=FutureWarning)
                 warnings.simplefilter(action="ignore", category=DeprecationWarning)
                 df = pa.deserialize(response.content)
+                if desired_resolution is None:
+                    desired_resolution = self.desired_resolution
+                if desired_resolution is not None:
+                    df = df.copy()
+                    if len(desired_resolution) != 3:
+                        raise ValueError(
+                            "desired resolution needs to be of length 3, for xyz"
+                        )
+                    vox_res = self.get_table_metadata(
+                        table, datastack_name, materialization_version
+                    )["voxel_resolution"]
+                    df = convert_position_columns(df, vox_res, desired_resolution)
+            if metadata:
+                attrs = self._assemble_attributes(
+                    tables,
+                    filters={
+                        "inclusive": filter_in_dict,
+                        "exclusive": filter_out_dict,
+                        "equal": filter_equal_dict,
+                        "spatial": filter_spatial_dict,
+                    },
+                    select_columns=select_columns,
+                    offset=offset,
+                    limit=limit,
+                    live_query=timestamp is not None,
+                    timestamp=string_format_timestamp(timestamp),
+                    materialization_version=materialization_version,
+                    desired_resolution=desired_resolution,
+                )
+                df.attrs.update(attrs)
             if split_positions:
                 return df
             else:
@@ -607,7 +686,6 @@ class MaterializatonClientV2(ClientBase):
         filter_out_dict=None,
         filter_equal_dict=None,
         filter_spatial_dict=None,
-        join_args=None,
         select_columns=None,
         offset: int = None,
         limit: int = None,
@@ -616,47 +694,49 @@ class MaterializatonClientV2(ClientBase):
         return_df: bool = True,
         split_positions: bool = False,
         materialization_version: int = None,
+        metadata: bool = True,
     ):
         """generic query on materialization tables
 
-        Args:
-            tables: list of lists with length 2 or 'str'
-                list of two lists: first entries are table names, second
-                                   entries are the columns used for the join
-            filter_in_dict (dict of dicts, optional):
-                outer layer: keys are table names
-                inner layer: keys are column names, values are allowed entries.
-                Defaults to None.
-            filter_out_dict (dict of dicts, optional):
-                outer layer: keys are table names
-                inner layer: keys are column names, values are not allowed entries.
-                Defaults to None.
-            filter_equal_dict (dict of dicts, optional):
-                outer layer: keys are table names
-                inner layer: keys are column names, values are specified entry.
-                Defaults to None.
-            filter_spatial (dict of dicts, optional):
-                outer layer: keys are table names:
-                inner layer: keys are column names, values are bounding boxes
-                             as [[min_x, min_y,min_z],[max_x, max_y, max_z]]
-                             Expressed in units of the voxel_resolution of this dataset.
-                Defaults to None
-            select_columns (list of str, optional): columns to select. Defaults to None.
-            offset (int, optional): result offset to use. Defaults to None.
-                will only return top K results.
-            limit (int, optional): maximum results to return (server will set upper limit, see get_server_config)
-            suffixes (list[str], optional): suffixes to use for duplicate columns same order as tables
-            datastack_name (str, optional): datastack to query.
-                If None defaults to one specified in client.
-            return_df (bool, optional): whether to return as a dataframe
-                default True, if False, data is returned as json (slower)
-            split_positions (bool, optional): whether to break position columns into x,y,z columns
-                default False, if False data is returned as one column with [x,y,z] array (slower)
-            materialization_version (int, optional): version to query.
-                If None defaults to one specified in client.
-
+         Args:
+             tables: list of lists with length 2 or 'str'
+                 list of two lists: first entries are table names, second
+                                    entries are the columns used for the join
+             filter_in_dict (dict of dicts, optional):
+                 outer layer: keys are table names
+                 inner layer: keys are column names, values are allowed entries.
+                 Defaults to None.
+             filter_out_dict (dict of dicts, optional):
+                 outer layer: keys are table names
+                 inner layer: keys are column names, values are not allowed entries.
+                 Defaults to None.
+             filter_equal_dict (dict of dicts, optional):
+                 outer layer: keys are table names
+                 inner layer: keys are column names, values are specified entry.
+                 Defaults to None.
+             filter_spatial (dict of dicts, optional):
+                 outer layer: keys are table names:
+                 inner layer: keys are column names, values are bounding boxes
+                              as [[min_x, min_y,min_z],[max_x, max_y, max_z]]
+                              Expressed in units of the voxel_resolution of this dataset.
+                 Defaults to None
+             select_columns (list of str, optional): columns to select. Defaults to None.
+             offset (int, optional): result offset to use. Defaults to None.
+                 will only return top K results.
+             limit (int, optional): maximum results to return (server will set upper limit, see get_server_config)
+             suffixes (list[str], optional): suffixes to use for duplicate columns same order as tables
+             datastack_name (str, optional): datastack to query.
+                 If None defaults to one specified in client.
+             return_df (bool, optional): whether to return as a dataframe
+                 default True, if False, data is returned as json (slower)
+             split_positions (bool, optional): whether to break position columns into x,y,z columns
+                 default False, if False data is returned as one column with [x,y,z] array (slower)
+             materialization_version (int, optional): version to query.
+                 If None defaults to one specified in client.
+             metadata: (bool, optional) : toggle to return metadata
+                 If True (and return_df is also True), return table and query metadata in the df.attr dictionary.
         Returns:
-            pd.DataFrame: a pandas dataframe of results of query
+             pd.DataFrame: a pandas dataframe of results of query
 
         """
         if materialization_version is None:
@@ -694,6 +774,24 @@ class MaterializatonClientV2(ClientBase):
                 warnings.simplefilter(action="ignore", category=DeprecationWarning)
                 df = pa.deserialize(response.content)
 
+            if metadata:
+                attrs = self._assemble_attributes(
+                    tables,
+                    suffixes=suffixes,
+                    filters={
+                        "inclusive": filter_in_dict,
+                        "exclusive": filter_out_dict,
+                        "equal": filter_equal_dict,
+                        "spatial": filter_spatial_dict,
+                    },
+                    select_columns=select_columns,
+                    offset=offset,
+                    limit=limit,
+                    live_query=False,
+                    timestamp=None,
+                    materialization_version=materialization_version,
+                )
+                df.attrs.update(attrs)
             if split_positions:
                 return df
             else:
@@ -798,7 +896,7 @@ class MaterializatonClientV2(ClientBase):
 
             del all_root_ids
             uniq_root_ids = uniq_root_ids[uniq_root_ids != 0]
-            logging.info(f"uniq_root_ids {uniq_root_ids}")
+            # logging.info(f"uniq_root_ids {uniq_root_ids}")
 
             is_latest_root = self.cg_client.is_latest_roots(
                 uniq_root_ids, timestamp=timestamp
@@ -819,11 +917,11 @@ class MaterializatonClientV2(ClientBase):
                     all_is_latest.append(is_latest_root)
                     n_svids = len(svids[~is_latest_root])
                     all_svid_lengths.append(n_svids)
-                    logging.info(f"{sv_col} has {n_svids} to update")
+                    logger.info(f"{sv_col} has {n_svids} to update")
                     all_svids = np.append(all_svids, svids[~is_latest_root])
-        logging.info(f"num zero svids: {np.sum(all_svids==0)}")
-        logging.info(f"all_svids dtype {all_svids.dtype}")
-        logging.info(f"all_svid_lengths {all_svid_lengths}")
+        logger.info(f"num zero svids: {np.sum(all_svids==0)}")
+        logger.info(f"all_svids dtype {all_svids.dtype}")
+        logger.info(f"all_svid_lengths {all_svid_lengths}")
         with TimeIt("get_roots"):
             # find the up to date root_ids for those supervoxels
             updated_root_ids = self.cg_client.get_roots(all_svids, timestamp=timestamp)
@@ -856,13 +954,15 @@ class MaterializatonClientV2(ClientBase):
         filter_out_dict=None,
         filter_equal_dict=None,
         filter_spatial_dict=None,
-        join_args=None,
         select_columns=None,
         offset: int = None,
         limit: int = None,
         datastack_name: str = None,
         split_positions: bool = False,
         post_filter: bool = True,
+        metadata: bool = True,
+        merge_reference: bool = True,
+        desired_resolution: Iterable = None,
     ):
         """generic query on materialization tables
 
@@ -900,6 +1000,14 @@ class MaterializatonClientV2(ClientBase):
                 So if, for example, a cell had a false merger split off since the last materialization.
                 those annotations on that incorrect portion of the cell will be included if this is False,
                 but will be filtered down if this is True. (Default=True)
+            metadata: (bool, optional) : toggle to return metadata
+                If True (and return_df is also True), return table and query metadata in the df.attr dictionary.
+            merge_reference: (bool, optional) : toggle to automatically join reference table
+                If True, metadata will be queries and if its a reference table it will perform a join
+                on the reference table to return the rows of that
+            desired_resolution: (Iterable[float], Optional) : desired resolution you want all spatial points returned in
+                If None, defaults to one specified in client, if that is None then points are returned
+                as stored in the table and should be in the resolution specified in the table metadata
 
         Returns:
         pd.DataFrame: a pandas dataframe of results of query
@@ -957,13 +1065,16 @@ class MaterializatonClientV2(ClientBase):
                 if len(past_equal_dict) == 0:
                     past_equal_dict = None
 
+        tables, suffixes = self._resolve_merge_reference(
+            merge_reference, table, datastack_name, materialization_version
+        )
         with TimeIt("package query"):
             url, data, query_args, encoding = self._format_query_components(
                 datastack_name,
                 materialization_version,
-                [table],
-                None,
+                tables,
                 None,
+                suffixes,
                 {table: past_filter_in_dict}
                 if past_filter_in_dict is not None
                 else None,
@@ -979,8 +1090,8 @@ class MaterializatonClientV2(ClientBase):
                 offset,
                 limit,
             )
-            logging.debug(f"query_args: {query_args}")
-            logging.debug(f"query data: {data}")
+            logger.debug(f"query_args: {query_args}")
+            logger.debug(f"query data: {data}")
         with TimeIt("query materialize"):
             response = self.session.post(
                 url,
@@ -995,11 +1106,26 @@ class MaterializatonClientV2(ClientBase):
             )
             self.raise_for_status(response)
 
+        if desired_resolution is None:
+            desired_resolution = self.desired_resolution
+
         with TimeIt("deserialize"):
             with warnings.catch_warnings():
                 warnings.simplefilter(action="ignore", category=FutureWarning)
                 warnings.simplefilter(action="ignore", category=DeprecationWarning)
                 df = pa.deserialize(response.content)
+                if desired_resolution is not None:
+                    df = df.copy()
+                    if len(desired_resolution) != 3:
+                        raise ValueError(
+                            "desired resolution needs to be of length 3, for xyz"
+                        )
+                    vox_res = self.get_table_metadata(
+                        table_name=table,
+                        datastack_name=datastack_name,
+                        version=materialization_version,
+                    )["voxel_resolution"]
+                    df = convert_position_columns(df, vox_res, desired_resolution)
             if not split_positions:
                 concatenate_position_columns(df, inplace=True)
         # post process the dataframe to update all the root_ids columns
@@ -1019,6 +1145,25 @@ class MaterializatonClientV2(ClientBase):
                 if filter_equal_dict is not None:
                     for col, val in filter_equal_dict.items():
                         df = df[df[col] == val]
+        if metadata:
+            attrs = self._assemble_attributes(
+                table,
+                join_query=False,
+                filters={
+                    "inclusive": filter_in_dict,
+                    "exclusive": filter_out_dict,
+                    "equal": filter_equal_dict,
+                    "spatial": filter_spatial_dict,
+                },
+                select_columns=select_columns,
+                offset=offset,
+                limit=limit,
+                live_query=timestamp is not None,
+                timestamp=string_format_timestamp(timestamp),
+                materialization_version=None,
+                desired_resolution=desired_resolution,
+            )
+            df.attrs.update(attrs)
 
         return df
 
@@ -1034,9 +1179,11 @@ class MaterializatonClientV2(ClientBase):
         limit: int = None,
         offset: int = None,
         split_positions: bool = False,
+        desired_resolution: Iterable[float] = None,
+        materialization_version: int = None,
         synapse_table: str = None,
         datastack_name: str = None,
-        materialization_version: int = None,
+        metadata: bool = True,
     ):
         """query synapses
 
@@ -1054,10 +1201,16 @@ class MaterializatonClientV2(ClientBase):
             offset (int, optional): number of synapses to offset query, Defaults to None (no offset).
             split_positions (bool, optional): whether to return positions as seperate x,y,z columns (faster)
                 defaults to False
+            desired_resolution : Iterable[float] or None, optional
+                If given, should be a list or array of the desired resolution you want queries returned in
+                useful for materialization queries.
             synapse_table (str, optional): synapse table to query. If None, defaults to self.synapse_table.
             datastack_name: (str, optional): datastack to query
             materialization_version (int, optional): version to query.
                 defaults to self.materialization_version if not specified
+            metadata: (bool, optional) : toggle to return metadata
+                If True (and return_df is also True), return table and query metadata in the df.attr dictionary.
+
         """
         filter_in_dict = {}
         filter_equal_dict = {}
@@ -1097,13 +1250,75 @@ class MaterializatonClientV2(ClientBase):
             offset=offset,
             limit=limit,
             split_positions=split_positions,
+            desired_resolution=desired_resolution,
             materialization_version=materialization_version,
             timestamp=timestamp,
+            datastack_name=datastack_name,
+            metadata=metadata,
+            merge_reference=False,
         )
+
+        if metadata:
+            df.attrs["remove_autapses"] = remove_autapses
+
         if remove_autapses:
             return df.query("pre_pt_root_id!=post_pt_root_id")
         else:
             return df
 
+    def _assemble_attributes(
+        self, tables, suffixes=None, desired_resolution=None, **kwargs
+    ):
+        if isinstance(tables, str):
+            tables = [tables]
+
+        join_query = len(tables) > 1
+
+        attrs = {
+            "datastack_name": self.datastack_name,
+        }
+        if not join_query:
+            attrs["join_query"] = False
+            meta = self.get_table_metadata(tables[0])
+            for k, v in meta.items():
+                if re.match("^table", k):
+                    attrs[k] = v
+                else:
+                    attrs[f"table_{k}"] = v
+            if desired_resolution is None:
+                attrs["dataframe_resolution"] = attrs["table_voxel_resolution"]
+            else:
+                attrs["dataframe_resolution"] = desired_resolution
+        else:
+            attrs["join_query"] = True
+            attrs["tables"] = {}
+            table_attrs = attrs["tables"]
+            if suffixes is None:
+                suffixes = ["_x", "_y"]
+            for (tname, jcol), s in zip(tables, suffixes):
+                table_attrs[tname] = {}
+                meta = self.get_table_metadata(tname)
+                for k, v in meta.items():
+                    if re.match("^table", k):
+                        table_attrs[tname][k] = v
+                    else:
+                        table_attrs[tname][f"table_{k}"] = v
+                table_attrs[tname]["join_column"] = jcol
+                table_attrs[tname]["suffix"] = s
+
+            if desired_resolution is None:
+                res = []
+                for tn in attrs["tables"]:
+                    res.append(attrs["tables"][tn]["table_voxel_resolution"])
+                if np.atleast_2d(np.unique(np.array(res), axis=0)).shape[0] == 1:
+                    attrs["dataframe_resolution"] = res[0]
+                else:
+                    attrs["dataframe_resolution"] = "mixed_resolutions"
+            else:
+                attrs["dataframe_resolution"] = desired_resolution
+
+        attrs.update(kwargs)
+        return attrs
+
 
 client_mapping = {2: MaterializatonClientV2, "latest": MaterializatonClientV2}
